{+--------------------------------------------------------------------------+
 | Component:   TmwCustomComboEdit
 | Created:     05.98
 | Author:      Martin Waldenburg
 | Copyright    1998, all rights reserved.
 | Description: Base of an universal Combo Control.
 | Status:      FreeWare
 | Version:     1.2
 | DISCLAIMER:  This is provided as is, expressly without a warranty of any kind.
 |              You use it at your own risc.
 +--------------------------------------------------------------------------+}
unit mwCombEdit;

interface

uses
  Windows, SysUtils, Messages, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, ExtCtrls, Buttons, Commctrl;

type
  TmwCustomComboEdit = class(TCustomEdit)
    Box: TSpeedButton;
    procedure BoxClick(Sender: TObject);
  private
    FCanvas: TControlCanvas;
    fImage: TBitmap;
    fStateImage: TBitmap;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure WndProc(var Message: TMessage); override;
    procedure SetStateImage(NewValue: TBitmap);
    procedure SetImage(NewValue: TBitmap);
    function GetBoxWidth:Integer;
    procedure SetBoxWidth(NewValue:Integer);
    function GetBoxGlyph:TBitmap;
    procedure SetBoxGlyph(NewValue:TBitmap);
  protected
    procedure CreateWindowHandle(const Params: TCreateParams); override;
    procedure SetEditRect;
    procedure SetBounds(Left, Top, Width, Height: Integer); override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
  public
    Action: procedure of object;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure DrawImage(Bitmap: TBitmap; Position: Integer);
  published
    property Font;
    property Text;
    property Image: TBitmap read fImage write SetImage;
    property StateImage: TBitmap read fStateImage write SetStateImage;
    property BoxWidth:Integer read GetBoxWidth write SetBoxWidth;
    property BoxGlyph:TBitmap read GetBoxGlyph write SetBoxGlyph;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('mw', [TmwCustomComboEdit]);
end; { Register }

destructor TmwCustomComboEdit.Destroy;
begin
  FCanvas.Free;
  inherited Destroy;
  FImage.free;
  FStateImage.free;
end; { Destroy }

constructor TmwCustomComboEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 141;
  Height := 21;
  TabOrder := 0;

  FCanvas := TControlCanvas.Create;
  FCanvas.Control := Self;

  Box := TSpeedButton.Create(Self);
  Box.Parent := Self;

  FImage := TBitmap.Create;
  FImage.Transparent := True;
  FStateImage := TBitmap.Create;
  FStateImage.Transparent := True;
end; { Create }

procedure TmwCustomComboEdit.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style or ES_MULTILINE or WS_CLIPCHILDREN;
end;

procedure TmwCustomComboEdit.CreateWnd;
begin
  inherited CreateWnd;
  SetEditRect;
end; { CreateWnd }

procedure TmwCustomComboEdit.CreateWindowHandle(const Params: TCreateParams);
begin
  inherited CreateWindowHandle(Params);

  with Box do
  begin
    Left := 120;
    Top := 0;
    Width := 17;
    Height := 17;
    Cursor := crArrow;
    OnClick := BoxClick;
  end; { Box }
end; { CreateWindowHandle }

procedure TmwCustomComboEdit.WMPaint(var Message: TWMPaint);
begin
  inherited PaintHandler(Message);
          if not FStateImage.Empty and not FImage.Empty then
          begin
            DrawImage(FStateImage, 3);
            DrawImage(FImage, FStateImage.Width + 6);
          end else
            if not FStateImage.Empty and FImage.Empty then
            begin
              DrawImage(FStateImage, 3);
            end else
              if FStateImage.Empty and not FImage.Empty then
              begin
                DrawImage(FImage, 3);
              end;
end; { WMPaint }

procedure TmwCustomComboEdit.WndProc(var Message: TMessage);
begin
  with Message do
    case Msg of
      WM_WINDOWPOSCHANGING: SetEditRect;
    end;
  inherited WndProc(Message);
end; { WndProc }

procedure TmwCustomComboEdit.SetEditRect;
var
  Loc: TRect;
begin
  if not FStateImage.Empty and not FImage.Empty then
  begin
    SetRect(Loc, FStateImage.Width + FImage.Width + 9, 0, ClientWidth - Box.Width - 2, ClientHeight + 1);
    SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@Loc));
  end else
    if not FStateImage.Empty and FImage.Empty then
    begin
      SetRect(Loc, FStateImage.Width + 6, 0, ClientWidth - Box.Width - 2, ClientHeight + 1);
      SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@Loc));
    end else
      if FStateImage.Empty and not FImage.Empty then
      begin
        SetRect(Loc, FImage.Width + 6, 0, ClientWidth - Box.Width - 2, ClientHeight + 1);
        SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@Loc));
      end else
        if FStateImage.Empty and FImage.Empty then
        begin
          SetRect(Loc, 0, 0, ClientWidth - Box.Width - 2, ClientHeight + 1);
          SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@Loc));
        end;
end;

procedure TmwCustomComboEdit.BoxClick(Sender: TObject);
begin
  if assigned(Action) then Action;
end; { BoxClick }

procedure TmwCustomComboEdit.SetBounds(Left, Top, Width, Height: Integer);
begin
  Case Parent <> nil of
    True:
      begin
        inherited SetBounds(Left, Top, Width, Height);
        with Box do
        begin
          Left := Self.Width - Box.Width - 4;
          Height := Self.Height - 4;
        end;
        SetEditRect;
      end;
    False: inherited SetBounds(Left, Top, Width, Height);
  end;
end; { SetBounds }

procedure TmwCustomComboEdit.DrawImage(Bitmap: TBitmap; Position: Integer);
var
  YPos: Integer;
begin
  YPos := ClientHeight div 2 - 7;
  FCanvas.Draw(Position, YPos, Bitmap);
end; { DrawImage }

procedure TmwCustomComboEdit.SetImage(NewValue: TBitmap);
begin
  FImage.Assign(NewValue);
  FImage.Transparent := True;
  SetEditRect;
  RePaint;
end; { SetImage }

procedure TmwCustomComboEdit.SetStateImage(NewValue: TBitmap);
begin
  FStateImage.Assign(NewValue);
  FStateImage.Transparent := True;
  SetEditRect;
  RePaint;
end; { SetStateImage }

function TmwCustomComboEdit.GetBoxWidth:Integer;
begin
  GetBoxWidth:= Box.Width;
end;   { GetBoxWidth }

procedure TmwCustomComboEdit.SetBoxWidth(NewValue:Integer);
begin
  Box.Width := NewValue;
  Box.Left := ClientWidth - Box.Width;
  SetEditRect;
end;   { SetBoxWidth }

function TmwCustomComboEdit.GetBoxGlyph:TBitmap;
begin
  GetBoxGlyph := Box.Glyph;
end;   { GetBoxGlyph }

procedure TmwCustomComboEdit.SetBoxGlyph(NewValue:TBitmap);
begin
  Box.Glyph := NewValue;
end;   { SetBoxGlyph }

end.

